
/*******************************************************************************
 Project name:  STM32_Termometer_DS18B20

 c/ Blue Pill (STM32F103C8) acoplado a um LCD 16x2

 C/ sensor de temperatura DS18B20

 Autor: Claudio Lrios

 Data:24/04/2021

 Compilador MikroC PRO for ARM V.6.2.0 (VERSO GRATUITA E LIMITADA)

 Use biblioteca ONE_WIRE,LCD,LCD_CONSTANTS nas opes da Library Manager.

 Freq=8MHZ INTERNA

 BLOG PICSOURCE

 Uso didtico apenas

 ******************************************************************************/


// LCD module connections
sbit LCD_RS at GPIOA_ODR.B5;
sbit LCD_EN at GPIOA_ODR.B4;
sbit LCD_D4 at GPIOA_ODR.B0;
sbit LCD_D5 at GPIOA_ODR.B1;
sbit LCD_D6 at GPIOA_ODR.B2;
sbit LCD_D7 at GPIOA_ODR.B3;
#define _GPIO_PINMASK_MY_LCD 0x3f

//  Ajuste TEMP_RESOLUTION para o correspondente sensor DS18x20 usado:
//  18S20: 9  (default setting; can be 9,10,11,or 12)
//  18B20: 12

const unsigned short TEMP_RESOLUTION = 12;   //DS18B20


unsigned temp_lida;

/*******************************************************************************
   Rotina de apresentao da temperatura no display LCD 16X2
*******************************************************************************/
void Mostra_Temp_LCD(unsigned int temp_lida_salva) {
  const unsigned short RES_SHIFT = TEMP_RESOLUTION - 8;
  char temp_int,sign;
  unsigned int temp_fracao,a,b;

  // Verifica se a temperatura  negativa
  if (temp_lida_salva & 0x8000) {

     sign = '-';
     temp_lida_salva = ~temp_lida_salva + 1;
  }
    else{
    sign = ' ';
    }

  // Extrai apenas a parte 'inteira' da temperatura
  temp_int = temp_lida_salva >> RES_SHIFT ;

   Lcd_Out(2,1,"Temp:           ");   // Escreve na linha 2
       Lcd_Cmd(0xc5);//2linha 5 coluna

       if(sign==1) Lcd_Chr_CP('-'); else Lcd_Chr_CP(' ');  //imprime sinal
       
       a = temp_int/100  + '0';
       temp_int = temp_int % 100;
        //verifica se  zero na casa de centenas
       if(a == '0')Lcd_Chr_CP(' '); // zero; envia somente espao
       else  {
        Lcd_Chr_CP(a); //imprime digito diferente de zero
        }
        

         b = temp_int/10  + '0';//obtm dezenas
         temp_int= temp_int%10;
       if(b == '0' & a == '0')Lcd_Chr_CP(' ');   // zero;envia espao
       else  {
        Lcd_Chr_CP(b);//envia dezenas
        }
        Lcd_Chr_CP(temp_int + '0');//envia unidades
        
       // Extrai apenas a parte 'fracionria' da temperatura
       temp_fracao  = temp_lida_salva << (4 - RES_SHIFT);
       temp_fracao &= 0x000F;  //limita a 4 bits
       temp_fracao *= 625; //multiplica por 625
       
       Lcd_Chr_Cp('.');   //imprime 'ponto' no LCD
       
       Lcd_Chr_Cp(temp_fracao/1000    + '0');//envia 1 casa ps-virgula
       temp_fracao= temp_fracao%1000;

       Lcd_Chr_Cp(temp_fracao/100 + '0');//envia 2 casa ps-virgula
       temp_fracao= temp_fracao%100;
       Lcd_Chr_Cp(temp_fracao/10 + '0');//envia 3 casa ps-virgula

       Lcd_Chr_CP(' ');
       Lcd_Chr_CP(0xdf);  // grau
       Lcd_Chr_CP('C');
       delay_ms(400);  //delay entre apresentaes  (determina taxa de apresentao)
  }
  
  void Le_Temperatura_ds18b20(){
       // Perform temperature reading
    Ow_Reset(&GPIOB_BASE, 12);// reseta Onewire
    Ow_Write(&GPIOB_BASE, 12, 0xCC);// Envia comando SKIP_ROM
    Ow_Write(&GPIOB_BASE, 12, 0x44);// Envia comando CONVERT_T
    Delay_ms(750);

    Ow_Reset(&GPIOB_BASE, 12);//reseta
    Ow_Write(&GPIOB_BASE, 12, 0xCC);// Envia comando SKIP_ROM
    Ow_Write(&GPIOB_BASE, 12, 0xBE);// Envia comando READ_SCRATCHPAD

    temp_lida =  Ow_Read(&GPIOB_BASE, 12); //l temperatura
    temp_lida = (Ow_Read(&GPIOB_BASE, 12) << 8) + temp_lida;
  }
/*******************************************************************************
   Rotina Main e Set ups
*******************************************************************************/
void main() {
  GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_MY_LCD); // Direo dos pinos LCD =sadas

  Lcd_Init();    // Inicializa o LCD
  Lcd_Cmd(_LCD_CURSOR_OFF);//desliga cursor
  Lcd_Cmd(_LCD_CLEAR);  //limpa a tela
  Lcd_Out(1,1,"Stm32_Termometer");   // Escreve nome do aparelho
  Lcd_Out(2,1," by PicSource");   // Escreve nome do blog
  delay_ms(2000);// Tempo de apresentao da mensagem
  
/*******************************************************************************
  Loop principal
*******************************************************************************/
  for(;;) {
    Le_Temperatura_ds18b20();//l sensor DS18B20
    Mostra_Temp_LCD(temp_lida);// Formata e mostra resultado no LCD 16x2
  }
}